'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var p = require('path');
var getReplacement = require('./get-replacement');

module.exports = codegenPlugin;

function codegenPlugin(_ref) {
  var transformFromAst = _ref.transformFromAst;

  return {
    name: 'codegen',
    visitor: {
      Program(path, _ref2) {
        var filename = _ref2.file.opts.filename;

        var firstNode = path.node.body[0] || {};
        var comments = firstNode.leadingComments || [];
        var isCodegen = comments.some(isCodegenComment);

        if (!isCodegen) {
          return;
        }

        comments.find(isCodegenComment).value = ' this file was codegened';

        var _transformFromAst = transformFromAst(path.node),
            string = _transformFromAst.code;

        var replacement = getReplacement({ string, filename });
        path.node.body = Array.isArray(replacement) ? replacement : [replacement];
      },
      TaggedTemplateExpression(path, _ref3) {
        var filename = _ref3.file.opts.filename;

        var isCodegen = path.node.tag.name === 'codegen';
        if (!isCodegen) {
          return;
        }
        var string = path.get('quasi').evaluate().value;
        if (!string) {
          throw new Error('Unable to determine the value of your codegen string');
        }
        var replacement = getReplacement({ string, filename });
        if (Array.isArray(replacement)) {
          path.replaceWithMultiple(replacement);
        } else {
          path.replaceWith(replacement);
        }
      },
      ImportDeclaration(path, _ref4) {
        var filename = _ref4.file.opts.filename;

        var isCodegen = looksLike(path, {
          node: {
            source: {
              leadingComments(comments) {
                return comments && comments.some(isCodegenComment);
              }
            }
          }
        });
        if (!isCodegen) {
          return;
        }
        var codegenComment = path.node.source.leadingComments.find(isCodegenComment).value.trim();
        var args = void 0;
        if (codegenComment !== 'codegen') {
          args = codegenComment.replace(/codegen\((.*)\)/, '$1').trim();
        }

        var replacement = getReplacement({
          string: `
            try {
              // allow for transpilation of required modules
              require('babel-register')
            } catch (e) {
              // ignore error
            }
            var mod = require('${path.node.source.value}');
            mod = mod && mod.__esModule ? mod.default : mod
            ${args ? `mod = mod(${args})` : ''}
            module.exports = mod
          `,
          filename
        });
        path.replaceWith(replacement);
      },
      CallExpression(path, _ref5) {
        var filename = _ref5.file.opts.filename;

        var isCodegen = looksLike(path, {
          node: {
            callee: {
              type: 'MemberExpression',
              object: { name: 'codegen' },
              property: { name: 'require' }
            }
          }
        });
        if (!isCodegen) {
          return;
        }

        var _path$get = path.get('arguments'),
            _path$get2 = _toArray(_path$get),
            source = _path$get2[0],
            args = _path$get2.slice(1);

        var string = resolveModuleToString({ args, filename, source });
        path.replaceWith(getReplacement.stringToAST(string));
      }
    }
  };
}

function resolveModuleToString(_ref6) {
  var args = _ref6.args,
      filename = _ref6.filename,
      source = _ref6.source;

  var argValues = args.map(function (a) {
    var result = a.evaluate();
    if (!result.confident) {
      throw new Error('codegen cannot determine the value of an argument in codegen.require');
    }
    return result.value;
  });
  var absolutePath = p.join(p.dirname(filename), source.node.value);
  try {
    // allow for transpilation of required modules
    require('babel-register');
  } catch (e) {
    // ignore error
  }
  var mod = require(absolutePath);
  mod = mod && mod.__esModule ? mod.default : mod;
  if (argValues.length) {
    if (typeof mod !== 'function') {
      throw new Error(`\`codegen.require\`-ed module (${source.node.value}) cannot accept arguments because it does not export a function. You passed the arguments: ${argValues.join(', ')}`);
    }
    mod = mod.apply(undefined, _toConsumableArray(argValues));
  }
  return mod;
}

function isCodegenComment(comment) {
  var normalisedComment = comment.value.trim().split(' ')[0].trim();
  return normalisedComment.startsWith('codegen') || normalisedComment.startsWith('@codegen');
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}

/*
eslint
  import/no-unassigned-import:0
  import/no-dynamic-require:0
*/